/******************************************************************************
 * VMPanel.java - Fill the fields in the About Box.
 * $Id: VMPanel.java,v 1.3 2011/06/04 01:50:13 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: VMPanel.java,v $
 * Revision 1.3  2011/06/04 01:50:13  dick
 * Get the version via Version.getVersion().
 *
 * Revision 1.2  2011/05/31 14:50:48  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.gui;

import java.awt.Color;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.util.JibsConvert;

import com.jeta.forms.components.panel.FormPanel;

/** Fill the fields in the About Box.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.3 $ <br> $Date: 2011/06/04 01:50:13 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/gui/VMPanel.java">cvs VMPanel.java</a>
 */
public class VMPanel extends FormPanel {
	private static final long serialVersionUID = -1548906758044612782L;
//	private JibsServer jibsServer;
	private JDialog frame;
	private JTextField java_version;
	private JTextField java_vm;
	private JTextField java_vm_version;
	private JTextField os_system;
	private JTextField os_version;
	private JTextField os_arch;
	private JTextField os_processors;
	private JTextField os_language;
	private JTextField totMemory;
	private JTextField freeMemory;
	private JTextField maxMemory;
	private JLabel aboutLabel;
	private JLabel lblBuild;
	private JButton btnOK;

	public VMPanel(JDialog dialog, JibsServer jibsServer, Frame oframe) {
		super("frm/vmPanel.jfrm");
		this.frame = dialog;
//		this.jibsServer = jibsServer;
		aboutLabel = getLabel("aboutLabel");
		java_version = getTextField("version");
		java_vm = getTextField("vm");
		java_vm_version = getTextField("vmversion");
		os_system = getTextField("os_system");
		os_version = getTextField("os_version");
		os_arch = getTextField("os_arch");
		os_processors = getTextField("os_processors");
		os_language = getTextField("os_language");
		totMemory = getTextField("totMemory");
		freeMemory = getTextField("freeMemory");
		maxMemory = getTextField("maxMemory");
		btnOK = (JButton) getButton("btnOk");
		lblBuild = getLabel("lblBuild");

		String buildDateString = "";

		try {
			Properties p = new Properties();
			InputStream is = ClassLoader.getSystemResourceAsStream("net/sourceforge/jibs/util/JibsConstants.properties");

			if (is != null) {
				p.load(is);
				buildDateString = p.getProperty("jibsBuild");
				lblBuild.setText("(" + buildDateString + ")");
			}
		} catch (IOException e) {
			jibsServer.logException(e);
		}

		frame.getRootPane().setDefaultButton(btnOK);
		btnOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				btn_OkActionPerformed(e);
			}
		});
		dialog.addWindowListener(new WindowListener() {
			public void windowActivated(WindowEvent e) {
			}

			public void windowClosed(WindowEvent e) {
			}

			public void windowClosing(WindowEvent e) {
				btn_OkActionPerformed(null);
			}

			public void windowDeactivated(WindowEvent e) {
			}

			public void windowDeiconified(WindowEvent e) {
			}

			public void windowIconified(WindowEvent e) {
			}

			public void windowOpened(WindowEvent e) {
			}
		});
	}

	public void doShow() {
		String v = net.sourceforge.jibs.util.Version.getVersion();
		aboutLabel.setText("jIBS-" + v);
		aboutLabel.setForeground(Color.RED);
		java_version.setText(System.getProperty("java.vm.version"));
		java_vm.setText(System.getProperty("java.vm.name"));
		java_vm_version.setText(System.getProperty("java.vm.info"));
		os_system.setText(System.getProperty("os.name"));
		os_version.setText(System.getProperty("os.version"));
		os_arch.setText(System.getProperty("os.arch"));
		os_processors.setText(Integer.toString(Runtime.getRuntime()
				.availableProcessors()));
		os_language.setText(System.getProperty("user.country"));

		double mem = Runtime.getRuntime().freeMemory() / (1024.0 * 1024.0);
		double x = JibsConvert.convdouble(mem, 3);
		String x1 = Double.toString(x);

		freeMemory.setText(x1 + " MB");
		mem = Runtime.getRuntime().maxMemory() / (1024.0 * 1024.0);
		x = JibsConvert.convdouble(mem, 3);
		x1 = Double.toString(x);
		maxMemory.setText(x1 + " MB");
		mem = Runtime.getRuntime().totalMemory() / (1024.0 * 1024.0);
		x = JibsConvert.convdouble(mem, 3);
		x1 = Double.toString(x);
		totMemory.setText(x1 + " MB");
	}

	public void btn_OkActionPerformed(ActionEvent event) {
		frame.dispose();
	}
}
