/******************************************************************************
 * Player.java - A Player
 * $Id: Player.java,v 1.15 2011/06/11 19:18:35 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Player.java,v $
 * Revision 1.15  2011/06/11 19:18:35  dick
 * Stop passing jibsServer all around.  If anybody wants it, they can do JibsServer.getInstance().
 *
 * Revision 1.14  2011/06/11 04:58:12  dick
 * Spell opponentPlayer correctly.
 *
 * Revision 1.13  2011/06/09 18:27:58  dick
 * getOutputStream is not public and goes away.
 * whoinfo(blah, blah, blah) which output to all, becomes getWhoinfo() which returns a string.
 * Fix whois() to whois(Player) and output that player to this player.
 * informToggleChange() becomes informWhoinfoChange() because that's what it does.
 *
 * Revision 1.12  2011/06/08 20:55:25  dick
 * Start match/game cleanups.
 * clip_ownInfo() doesn't need the jibsServer.
 *
 * Revision 1.11  2011/06/07 20:25:53  dick
 * Add startOfYourTurn().
 * Rename name to _name, just so it is first in the debugger attribute listing.
 *
 * Revision 1.10  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.9  2011/06/06 02:45:09  dick
 * Add setTogglesForNewGame().
 * Stop passing Server around.
 * is_valid() cleanups.
 *
 * Revision 1.8  2011/06/05 18:26:33  dick
 * log is a method of JibsServer.  Always write screen logs to the log file.
 * So, no need to pass write-to-logs=true from everywhere.
 *
 * Revision 1.7  2011/06/05 06:06:37  dick
 * mayDouble[12] becomes mayDouble[XO]
 *
 * Revision 1.6  2011/06/03 17:03:48  dick
 * getPlayerXPoints() becomes getPlayerXScore().
 *
 * Revision 1.5  2011/06/03 00:25:09  dick
 * Player.show2Watcher() becomes show2Watchers().
 *
 * Revision 1.4  2011/06/02 19:25:03  dick
 * Watching cleanups. - Notify the watchee that he is no longer being watched.
 *
 * Revision 1.3  2011/05/31 17:48:49  dick
 * Fix all warnings from the import.
 *
 * Revision 1.2  2011/05/31 15:25:49  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.io.BufferedReader;
import java.net.Socket;
import java.sql.Connection;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.TimeZone;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.util.ClipConstants;
import net.sourceforge.jibs.util.Encoder;
import net.sourceforge.jibs.util.JibsConvert;
import net.sourceforge.jibs.util.JibsSet;
import net.sourceforge.jibs.util.JibsToggle;
import net.sourceforge.jibs.util.JibsWriter;
import net.sourceforge.jibs.util.TimeSpan;

import org.apache.log4j.Logger;

import com.ibatis.sqlmap.client.SqlMapClient;

/** Define a player.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.15 $ <br> $Date: 2011/06/11 19:18:35 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/server/Player.java">cvs Player.java</a>
 */
public class Player {
	// ~ Static fields/initializers
	// ---------------------------------------------
	private static Logger logger = Logger.getLogger(Player.class);

	// ~ Instance fields
	// --------------------------------------------------------
	private ClientWorker cw;
	private int clipVersion;
	private JibsServer jibsServer;
	private BufferedReader in;
	private JibsWriter out;
	private TimeSpan idle;
	private JibsGame cur_Game;

	// important fields
	private boolean canCLIP;
	private String clientProgram;
	private Date last_login_date;
	private Date last_logout_date;
	private String last_login_host;
	private String _name;		// gdb sorts attributes alphabetically and I want this first.
	private boolean admin;
	private JibsQuestion question;
	private Player opponentPlayer;
	private Player watchingPlayer;
	private String password;
	private double rating;
	private String email;
	private int experience;
	private String strLastLogin;
	private String strLastHost;
	private JibsToggle jibsToggles;
	private JibsSet jibsSet;
	private int wavings;
	private HashMap<String, Player> watcher;
	private String awayMsg;
	private String toggle;
	private String boardstyle;
	private String linelength;
	private String pagelength;
	private String redoubles;
	private String sortwho;
	private String timezone;

	/** Empty constructor.  
	 * Used by jdbc.
	 */
	public Player() {}

	/** Constructor for New User
	 * @param jdbc
	 * @param out
	 * @param clientProgram
	 * @param clipVersion
	 * @param userName
	 * @param passWord
	 * @param rating
	 * @param experience
	 * @param email
	 * @param admin
	 * @param last_login
	 * @param last_host
	 */
	public Player(Connection jdbc,
			JibsWriter out, String clientProgram, int clipVersion,
			String userName, String passWord, double rating,
			int experience, String email, boolean admin, Date last_login,
			String last_host) {
		this.init(jdbc, out, clientProgram, clipVersion,
				userName, passWord, rating, experience, admin, last_login,
				last_host);
	}
	/** Constructor for a Player logging in.
	 * @param jdbc
	 * @param out
	 * @param clientProgram
	 * @param clipVersion
	 * @param strUser
	 * @param strPassword
	 */
	public Player(Connection jdbc,
			JibsWriter out, String clientProgram, int clipVersion,
			String strUser, String strPassword) {
		this.init(jdbc, out, clientProgram, clipVersion,
				strUser, strPassword, 1500.0, 0, false, new Date(), null);
	}

	// ~ Methods
	// ----------------------------------------------------------------
	private void init(Connection jdbc,
			JibsWriter out, String clientProgram, int ClipVersion,
			String strUser, String strPassword, double rating,
			int experience, boolean admin, Date last_login,
			String last_host) {
		this.jibsServer = JibsServer.getInstance();
		this.out = out;
		this.clientProgram = clientProgram;
		this.clipVersion = ClipVersion;
		this.canCLIP = ClipVersion >= 0;

		idle = new TimeSpan(0, 0);
		this.rating = rating;
		this.last_login_date = last_login;
		this.last_login_host = last_host;
		this._name = strUser;
		this.password = strPassword;

		jibsToggles = null;
		jibsSet = null;
		wavings = 0;
		watcher = null;
	}

	/** Get the number of games this Player has completed. 
	 * @return The number of completed games.
	 */
	public int getExperience() {
		return experience;
	}

	/** Set the reference to the JibsGame this Player is playing.
	 * @param game The JibsGame that we are part of.
	 */
	public void setGame(JibsGame game) {
		this.cur_Game = game;
	}

	/** Get the reference to the JibsGame this Player is playing.
	 * @return The JibsGame that we are part of. 
	 */
	public JibsGame getGame() {
		return(this.cur_Game);
	}

	/** Is this Player a jIBS admin?
	 * @return true if he is an admin.
	 */
	public boolean isAdmin() {
		return admin;
	}

	public void startGame(JibsGame game, int turn, 
			int mayDoubleX, int mayDoubleO) {
		BackgammonBoard board = game.getBackgammonBoard();
		game.startGame(turn, board.getXDie1(), board.getXDie2(),
				board.getPlayerXScore(), board.getPlayerOScore(),
				mayDoubleX, mayDoubleO);
	}

	/** Get a handle to our ClientWorker
	 * @return The ClientWorker thread
	 */
	public ClientWorker getClientWorker() {
		return cw;
	}

	public boolean canCLIP() {
		return canCLIP;
	}

	public void setOpponent(Player player) {
		opponentPlayer = player;
	}

	/** Return the player's name or "-" if he has no name
	 * @return The name or "-"
	 */
	public String getName() {
		return ((_name != null) && (_name.length() > 0)) ? _name : "-";
	}

	public String getPassword() {
		return password;
	}

	public double getRating() {
		return rating;
	}

	public String getTimezone() {
		return timezone;
	}

	/** Return the number of seconds that this player has been idle.
	 * @return The number of seconds
	 */
	public long getIdle() {
		long tend = idle.getEnd();
		long tstart = idle.getStart();

		return (tend - tstart) / 1000;
	}

	/** Return the opponent that this Player is playing against.
	 * @return The opponent, or null is this Player is not in a game.
	 */
	public Player getOpponent() {
		return opponentPlayer;
	}

	/** Return a Player that this player is watching.
	 * @return The Player we are watching, or null if this player isn't watching anyone.
	 */
	public Player getWatcher() {
		return watchingPlayer;
	}

	public String getClientProgram() {
		return clientProgram;
	}

	public String getUserName() {
		return _name;
	}

	public String getHostName() {
		return strLastHost;
	}

	public Date getLast_login_date() {
		return last_login_date;
	}

	public Date getLast_logout_date() {
		return last_logout_date;
	}

	public void setLast_logout_date(Date last_logout_date) {
		this.last_logout_date = last_logout_date;
	}

	public String getLastLoginHost() {
		return strLastHost;
	}

	public String getEmail() {
		if (email == null) {
			return "-";
		}

		return email;
	}

	public JibsServer getJibsServer() {
		return jibsServer;
	}

	public void setStreams(BufferedReader in, JibsWriter out) {
		this.in = in;
		this.out = out;
	}

	public void setJibsServer(JibsServer server) {
		this.jibsServer = server;
	}

	public void setIdle(TimeSpan idle) {
		this.idle = idle;
	}

	public BufferedReader getInputStream() {
		return in;
	}

	/*	public JibsWriter getOutputStream() {
		return out;
	}
	 */
	public void changeRating(double d, int exp) {
		this.rating = d;
		this.experience = exp;

		try {
			SqlMapClient sqlClient = jibsServer.getSqlMapInstance();
			sqlClient.update("Player.updateRating", this);
		} catch (SQLException e) {
			jibsServer.logException(e);
		}
	}

	public boolean changeAddress(String newadr) {
		try {
			this.email = newadr;

			SqlMapClient sqlClient = jibsServer.getSqlMapInstance();
			sqlClient.update("Player.updateMail", this);

			return true;
		} catch (Exception e) {
			jibsServer.logException(e);
		}

		return false;
	}

	public void clip_welcome() {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();

		// output connect information
		String strName = getName();
		String myDateStr = "-";

		// m_log_in=%0 logs in.
		Object[] obj = new Object[] { strName };
		String msg = jibsMessages.convert("m_log_in", obj);

		jibsServer.log(msg);

		/*
		 * Hmm, this searches for a single player that matches (case insensitive) this player.
		 * This could only be the player that was passed in.
		 * So why not just use it to begin with?
 		Map map = jibsServer.getServer().getAllClients();
		Set set = map.entrySet();
		Iterator iter = set.iterator();

		while (iter.hasNext()) {
			Entry entry = (Entry) iter.next();
			Player curPlayer = (Player) entry.getValue();

			if (!curPlayer.getName().equalsIgnoreCase(player.getName())) {
				String omsg = ClipConstants.CLIP_LOGIN + " " +
				player.getName() + " " + msg;
				curPlayer.getOutputStream().println(omsg);
			}
		}
		replace that foo with:
		 */
		String omsg = ClipConstants.CLIP_LOGIN + " " + getName() + " " + msg;
		//		out.println(omsg);
		this.informPlayers(omsg, null);


		if (last_login_date != null) {
			strLastLogin = Long.toString(last_login_date.getTime() / 1000);

			SimpleDateFormat formatter = new SimpleDateFormat("E MMMMMMMMMM dd HH:mm:ss yyyy z",
					Locale.US);
			TimeZone tz = TimeZone.getTimeZone(getTimezone());
			formatter.setTimeZone(tz);
			myDateStr = formatter.format(last_login_date);
		} else {
			myDateStr = "'Not known'";
		}

		if (last_login_host != null) {
			strLastHost = last_login_host;
		}

		if (canCLIP()) {
			out.println("");

			StringBuffer bf = new StringBuffer();

			bf.append(ClipConstants.CLIP_WELCOME + " " + strName + " " +
					strLastLogin + " " + strLastHost);
			out.println(bf.toString());
		} else {
			// m_welcome=Welcome %0.
			obj = new Object[] { strName };
			msg = jibsMessages.convert("m_welcome", obj);
			out.println(msg);

			// m_you_lastlogin=Your last login was %0 from %1.
			obj = new Object[] { myDateStr, getLastLoginHost() };
			msg = jibsMessages.convert("m_you_lastlogin", obj);
			out.println(msg);
		}
	}

	/** Send this message to every logged in player, except one
	 * @param msg The message to send to everyone.
	 * @param outsider The Player to not send the message to.
	 * 				   This is useful for "dick logs in." which you don't want to send to dick.
	 */
	public void informPlayers(String msg, Player outsider) {
		// send msg to all other player's except outsider
		//		Map map = jibsServer.getServer().getAllClients();
		//		Set set = map.entrySet();
		//		Iterator iter = set.iterator();

		//		while (iter.hasNext()) {
		//			Entry entry = (Entry) iter.next();
		//		Player curPlayer = (Player) entry.getValue();

		for (Player curPlayer : jibsServer.getAllClients().values()) {
			if (outsider == null) {
				if (!curPlayer.getName().equals(getName())) {
					if (curPlayer.checkToggle("notify")) {
						curPlayer.println(msg);
					}
				}
			} else {
				if (!curPlayer.getName().equals(getName()) &&
						!curPlayer.getName().equals(outsider.getName())) {
					if (curPlayer.checkToggle("notify")) {
						curPlayer.println(msg);
					}
				}
			}
		}
	}

	/** Out a whoinfo line about this player
	 * @param loginPlayer The Player we are talking about
	 */
	public String getWhoinfo() {
		StringBuffer bf = new StringBuffer();
		String opponent = "-";

		if (getOpponent() != null) {
			opponent = getOpponent().getName();
		}

		String watching = "-";

		if (getWatcher() != null) {
			watching = getWatcher().getName();
		}

		boolean ready = checkToggle("ready");
		boolean away = checkToggle("away");
		double rating = getRating();
		int experience = getExperience();
		long idleSecs = getIdle();
		Date login = getLast_login_date();
		String hostname = getHostName();
		String client = getClientProgram();
		String email = getEmail();


		bf.append(_name + " ");
		bf.append(opponent + " ");
		bf.append(watching + " ");
		bf.append(JibsConvert.convBoolean(ready) + " ");
		bf.append(JibsConvert.convBoolean(away) + " ");
		bf.append(JibsConvert.convdouble(rating, 2) + " ");
		bf.append(experience + " ");
		bf.append(idleSecs + " ");

		if (login == null) {
			long t = new java.util.Date().getTime();
			bf.append(t + " ");
		} else {
			bf.append(login.getTime());
			bf.append(" ");
		}

		if (hostname == null) {
			bf.append("- ");
		} else {
			bf.append(hostname + " ");
		}

		bf.append(client + " ");
		bf.append(email + " ");
		// out.println("aleucht - - 1 0 1558.84 27 0 1152787221306 localhost <DEB.eucht_________! - ");
		return(bf.toString());
	}

	public void whoinfo(Player player) {
		String s = player.getWhoinfo();
		if (canCLIP()) {
			out.print(ClipConstants.CLIP_WHO_INFO + " ");
			println(s);
			println(ClipConstants.CLIP_WHO_END + " ");
		}
	}

	public void whoinfoEnd() {
		if (canCLIP()) {
			out.println(ClipConstants.CLIP_WHO_END + " ");
		}
	}
	public void whois(Player p) {
		/*
		 * out.println("Information about bleucht");
		 * out.println(" Last login: Saturday, October 14 12:02 UTC from dslnet.85-22-11.ip187.dokom.de");
		 * out.println(" Last logout: Saturday, October 14 12:24 UTC");
		 * out.println(" Rating: 1457.75 Experience: 10");
		 */
		out.println("Information about " + p._name + ":");

		SimpleDateFormat formatter = new SimpleDateFormat("E MMMMMMMMMM dd HH:mm:ss yyyy z",
				Locale.US);
		TimeZone tz = TimeZone.getTimeZone(p.getTimezone());
		formatter.setTimeZone(tz);

		String myDateStr = formatter.format(p.last_login_date);
		out.println("Last login:  " + myDateStr);

		if (p.last_logout_date != null) {
			myDateStr = formatter.format(last_logout_date);
			out.println("Last logout: " + myDateStr);
		} else {
			out.println("Last logout: Not known");
		}

		if (jibsServer.isPlayerOnline(p)) {
			out.println("logged in right now.");
		} else {
			out.println("Not logged in right now.");
		}

		out.println("Rating: " + p.rating + " Experience: " + p.experience);

		if (p.email.equals("-")) {
			out.println("No email address.");
		} else {
			out.println("Email: " + email);
		}
	}

	/** Is this a valid user?  Flesh out this player from the database and 
	 * validate the password.
	 * @param strUser
	 * @param strPassword
	 * @return
	 */
	public boolean is_valid(String strUser, String strPassword) {
		boolean retCode = false;

		try {
			SqlMapClient sqlMap = jibsServer.getSqlMapInstance();
			Player retPlayer = (Player) sqlMap.queryForObject("Player.getPlayer",
					strUser);

			if (retPlayer != null) {
				_name = retPlayer.getName();
				password = retPlayer.getPassword();
				rating = retPlayer.getRating();
				experience = retPlayer.getExperience();
				email = retPlayer.getEmail();
				timezone = retPlayer.getTimezone();
				admin = retPlayer.isAdmin();
				jibsToggles = new JibsToggle(retPlayer.getToggle());
				jibsSet = new JibsSet(retPlayer.getBoardStyle(),
						retPlayer.getLineLength(),
						retPlayer.getPageLength(),
						retPlayer.getRedoubles(),
						retPlayer.getSortwho(),
						retPlayer.getTimezone());

				if (email.equals("")) {
					email = "-";
				}

				last_login_date = retPlayer.getLast_login_date();
				last_logout_date = retPlayer.getLast_logout_date();
				last_login_host = retPlayer.getLast_login_host();

				if (_name.equals(strUser)) {
					if (password.equals(Encoder.encrypt(strPassword, "MD5"))) {
						retCode = true;
					} else {
						String conString = "Connect unknown password" + "(" +
						strUser + "," + strPassword + ")";

						jibsServer.log(conString);
					}
				} else {
					String conString = "Connect unknown user" + "(" + strUser +
					"," + strPassword + ")";

					jibsServer.log(conString);
					logger.warn(conString);
				}
			} else {
				String conString = "Connect unknown user" + "(" + strUser +
				"," + strPassword + ")";

				jibsServer.log(conString);
				logger.warn(conString);
			}
		} catch (SQLException sqle) {
			jibsServer.logException(sqle);
			retCode = false;
		}

		return retCode;
	}

	public void setName(String string) {
		this._name = string;
	}

	public void setPassword(String string) {
		this.password = string;
	}

	public void setEmail(String string) {
		this.email = string;
	}

	public void setRating(double double1) {
		this.rating = double1;
	}

	public void setExperience(int int1) {
		this.experience = int1;
	}

	public void setLastHost(String string) {
		this.strLastHost = string;
	}

	public void setLastLogin(Date timestamp) {
		if (timestamp != null) {
			this.last_login_date = timestamp;
		} else {
			this.last_login_date = null;
		}
	}

	public void setAdmin(int boolean1) {
		this.admin = boolean1 > 0;
	}

	public void setQuestion(JibsQuestion question) {
		this.question = question;
	}

	public void ask(JibsQuestion question) {
		this.question = question;
	}

	public JibsQuestion getQuestion() {
		return question;
	}

	public void endGame() {

		setOpponent(null);
		this.informWhoinfoChange();
	}

	public void setClientWorker(ClientWorker cw2) {
		this.cw = cw2;
	}

	public void setClientProgram(String clientProgram) {
		this.clientProgram = clientProgram;
	}

	public JibsSet getJibsSet() {
		return jibsSet;
	}

	public Socket getSocket() {
		if (cw != null) {
			return cw.getSocket();
		} else {
			return null;
		}
	}

	public void canCLIP(boolean b) {
		this.canCLIP = b;
	}

	public JibsToggle getJibsToggles() {
		return jibsToggles;
	}

	public boolean checkToggle(String string) {
		return jibsToggles.get(string).booleanValue();
	}

	public int getWavings() {
		return wavings;
	}

	public void setWavings(int wavings) {
		this.wavings = wavings;
	}

	/** The player's whoinfo changed.  Tell everybody about it.
	 */
	public void informWhoinfoChange() {
		for (Player curPlayer : jibsServer.getAllClients().values()) {
			curPlayer.whoinfo(this);
			curPlayer.whoinfoEnd();
		}
	}

	/** Send the CLIP Own Info line to this player 
	 */
	public void clip_ownInfo() {
		StringBuffer bf = new StringBuffer();

		bf.append(ClipConstants.CLIP_OWN_INFO);
		bf.append(" ");
		bf.append(getName());
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("allowpip")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("autoboard")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("autodouble")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("automove")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("away")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("bell")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("crawford")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("double")));
		bf.append(" ");
		bf.append(experience);
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("greedy")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("moreboards")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("moves")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("notify")));
		bf.append(" ");
		bf.append(JibsConvert.convdouble(rating, 2));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("ratings")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("ready")));
		bf.append(" ");
		bf.append("0"); // redoubles
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("report")));
		bf.append(" ");
		bf.append(JibsConvert.convBoolean(checkToggle("silent")));
		bf.append(" ");
		bf.append("UTC"); // Timezone

		if (canCLIP()) {
			println(bf.toString());
		}
	}

	/** Another Player is watching this Player play
	 * @param player The Player who is doing the watching.
	 */
	public void addWatcher(Player player) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String msg = null;

		if (watcher == null) {
			watcher = new HashMap<String, Player>();
		}

		if (!watcher.containsKey(player.getName())) {
			watcher.put(player.getName(), player);

			// m_you_watch=You're now watching %0.
			Object[] obj = new Object[] { getName() };
			msg = jibsMessages.convert("m_you_watch", obj);
			player.println(msg);
			// m_other_watch=%0 is watching you.
			obj = new Object[] { player.getName() };
			msg = jibsMessages.convert("m_other_watch", obj);
			out.println(msg);
		}
	}

	/** Show this board to our watchers
	 * @param board The board to show
	 * @param name The name to use for PlayerX (may be "You", but no not really never)
	 * @param i die
	 * @param j die
	 * @param k die
	 * @param l die
	 */
	public void show2WatcherBoard(BackgammonBoard board, String name, int i,
			int j, int k, int l) {
		if (watcher != null) {
			for (Player p : watcher.values()) {

				if (p != null) {
					board.outBoard(p, name, board.getTurn(), i, j, k, l);
				}
			}
		}
	}

	/** Get this Player's "Away" message
	 * @return The away message
	 */
	public String getAwayMsg() {
		return awayMsg;
	}

	/** Set this Player's "Away" message
	 * @param awayMsg The away message to set
	 */
	public void setAwayMsg(String awayMsg) {
		this.awayMsg = awayMsg;
	}

	/** Send this message to anyone watching this player.
	 * @param msg The message to send.
	 */
	public void show2Watchers(String msg) {
		if (watcher != null) {
			for (Player curPlayer : watcher.values()) {
				curPlayer.println(msg);
			}
		}
	}

	/** If this Player is watching someone else play, return the watched Player
	 * @return The Player we are watching
	 */
	public Player getWatchingPlayer() {
		return watchingPlayer;
	}

	/** This Player has started watching someone else play
	 * @param watchingPlayer The Player we are watching
	 */
	public void setWatchingPlayer(Player watchingPlayer) {
		this.watchingPlayer = watchingPlayer;
	}

	/** If this Player is watching another Player, then turn it off.
	 */
	public void stopWatching() {
		if (this.watchingPlayer != null) {
			this.watchingPlayer.stopWatching(this);
		}
		this.watchingPlayer = null;
	}

	/** If other Player is watching this player, then turn it off <br>
	 * this = the Player being watched
	 * @param player The Player doing the watching
	 */
	public void stopWatching(Player player) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String msg = null;

		if (watcher != null && watcher.containsKey(player._name)) {
			watcher.remove(player._name);
			// m_you_stop_watch=You stop watching %0.
			Object[] obj = new Object[] { getName() };
			msg = jibsMessages.convert("m_you_stop_watch", obj);
			player.println(msg);

			// m_other_watch_stop=%0 stops watching you.
			obj = new Object[] { player.getName() };
			msg = jibsMessages.convert("m_other_watch_stop", obj);
			out.println(msg);
		}
	}


	public int informWatchers(String string, Object[] obj, boolean doConvert) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		String msg = null;
		int heard = 0;

		if (watcher != null) {
			for (Player wPlayer : watcher.values()) {

				if (doConvert) {
					msg = jibsMessages.convert(string, obj);
				} else {
					msg = string;
				}

				wPlayer.println(msg);
				heard++;
			}
		}

		return heard;
	}

	/** Change this Player's password in the database.
	 * @param newPassword1 The new password to set
	 */
	public void changePassword(String newPassword1) {
		this.password = newPassword1;

		SqlMapClient sqlClient = jibsServer.getSqlMapInstance();

		try {
			sqlClient.update("Player.updatePassword", this);
		} catch (SQLException e) {
			jibsServer.logException(e);
		}
	}

	/** Change one of the JibsToggle attributes
	 * @param string The name of one of the predefined toggles.  Note: No range checking done on this name.
	 * @param value The new state of the toggle.
	 */
	public void changeToggle(String string, Boolean value) {
		jibsToggles.getToggleMap().put(string, value);
	}

	/** Set the state of the toggles for a new game. 
	 */
	public void setTogglesForNewGame() {
		changeToggle("double", true);
		changeToggle("greedy", false);
	}

	public void setLast_login_date(Date last_login_date) {
		this.last_login_date = last_login_date;
	}

	public String getLast_login_host() {
		return last_login_host;
	}

	public void setLast_login_host(String last_login_host) {
		this.last_login_host = last_login_host;
	}

	public String getBoardStyle() {
		return boardstyle;
	}

	public void setBoardStyle(String boardStyle) {
		this.boardstyle = boardStyle;
	}

	public String getLineLength() {
		return linelength;
	}

	public void setLineLength(String lineLength) {
		this.linelength = lineLength;
	}

	public String getPageLength() {
		return pagelength;
	}

	public void setPageLength(String pageLength) {
		this.pagelength = pageLength;
	}

	public String getRedoubles() {
		return redoubles;
	}

	public void setRedoubles(String redoubles) {
		this.redoubles = redoubles;
	}

	public String getSortwho() {
		return sortwho;
	}

	public void setSortwho(String sortwho) {
		this.sortwho = sortwho;
	}

	public String getToggle() {
		return toggle;
	}

	public void setToggle(String toggle) {
		this.toggle = toggle;
	}

	public void setTimezone(String timezone) {
		this.timezone = timezone;
	}

	public int getClipVersion() {
		return clipVersion;
	}

	public void setClipVersion(int clipVersion) {
		this.clipVersion = clipVersion;
	}

	/** Convienence function that calls JibsWriter directly
	 * @param s
	 */
	public void println(String s) {
		out.println(s);
	}

	/** It's the start of this player's turn.  Either roll the dice or ask what he wants to do.
	 * @param board The board that this player is playing on.
	 */
	public void startOfYourTurn(BackgammonBoard board) {
		String msg;
		JibsMessages jibsMessages = jibsServer.getJibsMessages();

		if (checkToggle("double")) {
			if (board.getMayDouble(this)) {
				// m_roll_or_double=It's your turn. Please roll or double
				msg = jibsMessages.convert("m_roll_or_double");
				println(msg);
			} else {
				getClientWorker().executeCmd("roll");
			}
		} else {
			board.getXDie1().setValue(0);
			board.getXDie2().setValue(0);
			getClientWorker().executeCmd("roll");
		}

	}
}
